@@ -468,6 +468,68 @@ class Database:
             df = pd.concat(df_list)
             
             
+            df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
+            df = df.set_index(keys='date')
+            df = df.sort_index()
+
+            return df
+        else:
+            err_msg = 'Error in: '
+            for error in check_bool_dict:
+                if check_bool_dict[error] == False:
+                    err_msg += error + ','
+            print(err_msg)
+
+    def get_hk_shortsell_by_code(self, code, agg_value = False):
+
+        link_url = 'http://www.hkfdb.net/data_api?'
+        token_str = 'token=' + str(self.authToken)
+        database_str = 'database=' + 'hk_shortsell_by_code'
+        code_str = 'code=' + code
+        link_str = link_url + code_str + '&' + token_str + '&' + database_str
+
+        response = requests.get(link_str)
+        result = json.loads(json.loads(response.content).replace("'", "\""))
+
+        df = pd.DataFrame(result)
+        df['date'] = pd.to_datetime(df['date'], format = '%Y%m%d')
+        df = df.set_index(keys='date')
+        df = df.sort_index()
+
+        if agg_value == True:
+            df = df[['value']]
+            df = df.groupby(df.index).agg({'value': sum})
+
+        return df
+
+    def get_hk_shortsell_by_date(self, start_date, end_date):
+
+        check_bool_dict = self.check_start_end_date(start_date, end_date)
+
+        if False not in list(check_bool_dict.values()):
+
+            link_url = 'http://www.hkfdb.net/data_api?'
+            token_str = 'token=' + str(self.authToken)
+            database_str = 'database=' + 'hk_shortsell_by_date'
+            start_date_str = 'start_date=' + str(start_date)
+            end_date_str = 'end_date=' + str(end_date)
+            link_str = link_url + '&' + token_str + '&' + database_str + '&' + start_date_str + '&' + end_date_str
+
+            response = requests.get(link_str)
+            json_content = json.loads(response.content)
+            json_content = json_content.replace(' nan', '\" nan\"')
+
+            result = ast.literal_eval(json_content)
+
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
+
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
             df = df.set_index(keys='date')
             df = df.sort_index()
@@ -867,15 +929,22 @@ class Database:
                     df[col] = pd.to_datetime(df[col], format='%Y-%m-%d')
             
             if index_name == 'hsi_const_hist':
-                end_date_list = df['end_date'].to_list()
-                latest_date = max(set(end_date_list), key=end_date_list.count)
+                mode = df['end_date'].mode()[0]
+                df['is_alive'] = df['is_alive'].astype(bool)
+                df['is_delisted'] = df['is_delisted'].astype(bool)
+                df = df.rename(columns={'is_alive':'is_active'})
+
                 for i in range(len(df)):
-                    end_date = df.loc[i,'end_date']
-                    if end_date == latest_date:
-                        last_sunday = datetime.datetime.now() - datetime.timedelta(days=((datetime.datetime.now().isoweekday()) % 7))
-                        df.at[i, 'end_date'] = last_sunday.date()
-                    else:
-                        df.at[i, 'end_date'] = end_date.date()
+                    end_date = df.loc[i, 'end_date']
+                    if end_date == mode:
+                        df.at[i, 'end_date'] = datetime.datetime.now().strftime('%Y-%m-%d')
+                for col in df.columns:
+                    if col == 'start_date' or col == 'end_date':
+                        df[col] = pd.to_datetime(df[col], format='%Y-%m-%d')
+
+            if index_name == 'hk_opt_stocks':
+                df = df.rename(columns={'ATS_code':'code'})
+
 
             return df
 
